package Visitors;
import AST.*;

/**
 * Visitor to construct a string that represents the subexpression
 * of the subtree for which the current node is the head. 
 * 
 * This applies only to nodes that are part of expressions, i.e.
 * IdNode, AddOpNode, MultOpNode, and AssignStatNode. 
 * 
 * Note that this is just as an example. Such functionality is not
 * required in the project. 
 * 
 * However, note that this is essentially how the code generation phase
 * will eventually proceed. 
 * 
 */

public class ConstructAssignmentAndExpressionStringVisitor  extends Visitor {
	
	public void visit(IdNode node){ 
		System.out.println("ConstructAssignmentAndExpressionStringVisitor Visiting IdNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		// Then, do the processing of this nodes' visitor
		node.setSubtreeString(node.getData());
	}
	
	public void visit(NumNode node){ 
		System.out.println("ConstructAssignmentAndExpressionStringVisitor Visiting NumNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		// Then, do the processing of this nodes' visitor
		node.setSubtreeString(node.getData());
	}

	public void visit(AddOpNode node){
		System.out.println("ConstructAssignmentAndExpressionStringVisitor Visiting AddOpNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		// Then, do the processing of this nodes' visitor
		node.setSubtreeString(	node.getChildren().get(0).getSubtreeString() + 
								node.getData() +
								node.getChildren().get(1).getSubtreeString() );
	}

	public void visit(MultOpNode node){ 
		System.out.println("ConstructAssignmentAndExpressionStringVisitor Visiting MultOpNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		// Then, do the processing of this nodes' visitor
		node.setSubtreeString(	node.getChildren().get(0).getSubtreeString() + 
								node.getData() +
								node.getChildren().get(1).getSubtreeString() );
	}
	
	public void visit(AssignStatNode node){ 
		System.out.println("ConstructAssignmentAndExpressionStringVisitor Visiting AssignStatNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		// Then, do the processing of this nodes' visitor
		node.setSubtreeString(	node.getChildren().get(0).getSubtreeString() + 
								node.getData() +
								node.getChildren().get(1).getSubtreeString() );
	}

	// Below are the visit methods for node types for which this visitor does
	// not apply. They still have to propagate acceptance of the visitor to
	// their children.
	public void visit(ClassListNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(ClassNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(DimListNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(FuncDefListNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(FuncDefNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(Node node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(ProgNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(ProgramBlockNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(PutStatNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(StatBlockNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(TypeNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(VarDeclNode node) {
			for (Node child : node.getChildren() )
				child.accept(this);
	 }; 
}
