package Visitors;
import AST.*;

/**
 * Visitor to compute the type of subexpressions and assignment statements. 
 * 
 * This applies only to nodes that are part of expressions and assignment statements i.e.
 * AddOpNode, MultOpNode, and AssignStatNode. 
 * 
 */

public class TypeCheckingVisitor extends Visitor {

	public void visit(AddOpNode node){
		System.out.println("TypeCheckingVisitor Visiting AddOppNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		String leftOperandType  = node.getChildren().get(0).getType();
		String rightOperandType = node.getChildren().get(1).getType();
		if( leftOperandType == rightOperandType )
			node.setType(leftOperandType);
		else{
			node.setType("typeerror");
			System.out.println("TYPE ERROR DETECTED between " 
					+ node.getChildren().get(0).getData()
					+  " and "
					+ node.getChildren().get(1).getData()
					);
		}
	}

	public void visit(MultOpNode node){ 
		System.out.println("TypeCheckingVisitor Visiting MultOpNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		String leftOperandType  = node.getChildren().get(0).getType();
		String rightOperandType = node.getChildren().get(1).getType();
		if( leftOperandType == rightOperandType )
			node.setType(leftOperandType);
		else{
			node.setType("typeerror");
			System.out.println("TYPE ERROR DETECTED between " 
								+ node.getChildren().get(0).getData()
								+  " and "
								+ node.getChildren().get(1).getData()
								);
		}
	}
	
	public void visit(AssignStatNode node){ 
		System.out.println("TypeCheckingVisitor Visiting AssignStatNode");
		// First, propagate accepting the same visitor to all the children
		// This effectively achieves Depth-First AST Traversal
		for (Node child : node.getChildren() )
			child.accept(this);
		String leftOperandType  = node.getChildren().get(0).getType();
		String rightOperandType = node.getChildren().get(1).getType();
		if( leftOperandType == rightOperandType )
			node.setType(leftOperandType);
		else{
			node.setType("typeerror");
			System.out.println("TYPE ERROR DETECTED between " 
								+ node.getChildren().get(0).getData()
								+  " and "
								+ node.getChildren().get(1).getData()
								);
		}
	}
	
	// Below are the visit methods for node types for which this visitor does not apply
	// They still have to propagate acceptance of the visitor to their children.
	public void visit(ClassListNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(ClassNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(DimListNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(FuncDefListNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(FuncDefNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(IdNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(Node node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(NumNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(ProgNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(ProgramBlockNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(PutStatNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(StatBlockNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(TypeNode node) {
		for (Node child : node.getChildren())
			child.accept(this);
	};

	public void visit(VarDeclNode node) {
			for (Node child : node.getChildren() )
				child.accept(this);
	 }; 
}
