package Visitors;
import AST.*;

/**
 * Visitor superclass. Can be either an interface or an abstract class. 
 * Needs to have one visit method for each of the visit methods 
 * implemented by any of its subclasses. 
 * 
 * This forces all its subclasses to implement all of them, even if they 
 * are not concerned with processing of this particular subtype, creating 
 * visit methods with a body whose only function is to propagate accept() to
 * all the children of the visited node.  
 */
 
 public abstract class Visitor {
	 public abstract void visit(AddOpNode        node);
	 public abstract void visit(AssignStatNode   node);
	 public abstract void visit(ClassListNode    node);
	 public abstract void visit(ClassNode        node);
	 public abstract void visit(DimListNode      node);
	 public abstract void visit(FuncDefListNode  node);  
	 public abstract void visit(FuncDefNode      node);
	 public abstract void visit(IdNode           node);
	 public abstract void visit(MultOpNode       node); 
	 public abstract void visit(Node             node);
	 public abstract void visit(NumNode          node);
	 public abstract void visit(ProgNode         node);
	 public abstract void visit(ProgramBlockNode node);
	 public abstract void visit(PutStatNode      node);
	 public abstract void visit(StatBlockNode    node);
	 public abstract void visit(TypeNode         node);
	 public abstract void visit(VarDeclNode      node); 
}